# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #


class WarningFlags:

    """Critical System Error."""
    CRITICAL_SYSTEM_ERROR = "FF"

    """Overvoltage or Undervoltage."""
    OVERVOLTAGE_OR_UNDERVOLTAGE = "FV"

    """Driver Disabled."""
    DRIVER_DISABLED = "FD"

    """Encoder Error."""
    ENCODER_ERROR = "FQ"

    """Excessive Twist."""
    EXCESSIVE_TWIST = "FT"

    """Index Error."""
    INDEX_ERROR = "FI"

    """Analog Encoder Sync Error."""
    ANALOG_ENCODER_SYNC_ERROR = "FA"

    """Stalled and Stopped."""
    STALLED_AND_STOPPED = "FS"

    """Stream Bounds Error."""
    STREAM_BOUNDS_ERROR = "FB"

    """Interpolated Path Deviation."""
    INTERPOLATED_PATH_DEVIATION = "FP"

    """Limit Error."""
    LIMIT_ERROR = "FE"

    """Unexpected Limit Trigger."""
    UNEXPECTED_LIMIT_TRIGGER = "WL"

    """Voltage Out of Range."""
    VOLTAGE_OUT_OF_RANGE = "WV"

    """Controller Temperature High."""
    CONTROLLER_TEMPERATURE_HIGH = "WT"

    """Stalled with Recovery."""
    STALLED_WITH_RECOVERY = "WS"

    """Displaced When Stationary."""
    DISPLACED_WHEN_STATIONARY = "WM"

    """No Reference Position."""
    NO_REFERENCE_POSITION = "WR"

    """Device Not Homed."""
    DEVICE_NOT_HOMED = "WH"

    """Invalid Calibration Type."""
    INVALID_CALIBRATION_TYPE = "WP"

    """Manual Control."""
    MANUAL_CONTROL = "NC"

    """Movement Interrupted."""
    MOVEMENT_INTERRUPTED = "NI"

    """Stream Discontinuity."""
    STREAM_DISCONTINUITY = "ND"

    """Value Rounded."""
    VALUE_ROUNDED = "NR"

    """Joystick Calibrating."""
    JOYSTICK_CALIBRATING = "NJ"

    """Setting Update Pending."""
    SETTING_UPDATE_PENDING = "NU"
