# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
from enum import Enum


class RotationDirection(Enum):
    """
    Direction of rotation.
    """

    CLOCKWISE = 0
    COUNTERCLOCKWISE = 1
    CW = 0
    CCW = 1
